/// <reference path='types.d.ts' />
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
/**
 * TypeScript dependencies.
 */
var AbstractCommand = require('abstract-command');
var closest = require('component-closest');
var RangeIterator = require('range-iterator');
var blockElements = require('block-elements');
var contains = require('node-contains');
var DEBUG = require('debug');
var debug = DEBUG('indent-command');
var blockSel = blockElements.join(', ');
/**
 * `IndentCommand` class wraps a BLOCKQUOTE element around the given `Range`.
 *
 * ``` js
 * var indent = new IndentCommand();
 * if (indent.queryEnabled()) {
 *   indent.execute();
 * }
 * ```
 *
 * @public
 */
var IndentCommand = (function (_super) {
    __extends(IndentCommand, _super);
    function IndentCommand(root, doc) {
        if (root === void 0) { root = document.documentElement; }
        if (doc === void 0) { doc = root.ownerDocument; }
        _super.call(this, doc);
        this.root = root;
        debug('created IndentCommand: document %o', this.document);
    }
    IndentCommand.prototype.createElement = function () {
        return this.document.createElement('blockquote');
    };
    IndentCommand.prototype.wrapNode = function (node, blockquote, blocks) {
        var block = closest(node, blockSel, true, this.root);
        debug('closest "block" node: %o', block);
        if (block == this.root || !contains(this.root, block)) {
            debug('sanity check failed, "block" node is not inside the specified root. skipping');
            return;
        }
        if (block && -1 === blocks.indexOf(block)) {
            blocks.push(block);
            // add BLOCKQUOTE element to the DOM, only once
            if (!blockquote.parentNode) {
                block.parentNode.insertBefore(blockquote, block);
            }
            blockquote.appendChild(block);
        }
    };
    IndentCommand.prototype._execute = function (range, value) {
        var _this = this;
        // array to ensure that we only process a particular block node once
        // (in the instance that it has multiple text node children)
        var blocks = [];
        var common = range.commonAncestorContainer;
        var startContainer = range.startContainer;
        var startOffset = range.startOffset;
        var endContainer = range.endContainer;
        var endOffset = range.endOffset;
        var blockquote = this.createElement();
        if (range.collapsed) {
            this.wrapNode(range.endContainer, blockquote, blocks);
        }
        else {
            var next;
            var iterator = RangeIterator(range, function (node) { return (node.childNodes.length === 0 && contains(_this.root, node)); });
            while (!(next = iterator.next()).done) {
                this.wrapNode(next.value, blockquote, blocks);
            }
        }
        if (startContainer.firstChild === blockquote) {
            startContainer = blockquote;
        }
        if (endContainer.firstChild === blockquote) {
            endContainer = blockquote;
        }
        range.setStart(startContainer, startOffset);
        range.setEnd(endContainer, endOffset);
    };
    IndentCommand.prototype._queryState = function (range) {
        var _this = this;
        var blockquote;
        if (range.collapsed) {
            blockquote = closest(range.endContainer, 'blockquote', true, this.root);
            return !!blockquote;
        }
        else {
            var next;
            var count = 0;
            var iterator = RangeIterator(range, function (node) { return (node.childNodes.length === 0 && contains(_this.root, node)); });
            while (!(next = iterator.next()).done) {
                count++;
                blockquote = closest(next.value, 'blockquote', true, this.root);
                if (!blockquote)
                    return false;
            }
            return count > 0;
        }
    };
    return IndentCommand;
})(AbstractCommand);
module.exports = IndentCommand;
